@@ -19,7 +19,7 @@
 #'  c(14, "bold", "red")}.  Use font.x = 14, to change only font size; or use
 #'  font.x = "bold", to change only font face.
 #'@param ggtheme function, ggplot2 theme name. Default value is \link{theme_classic2}.
-#'  Allowed values include ggplot2 official themes: see \link{ggtheme}.
+#'  Allowed values include ggplot2 official themes: see \code{\link[ggplot2]{theme}}.
 #'@return Returns an object of class \code{ggcoxzph} which is a list of ggplots.
 #'
 #'@author Marcin Kosinski , \email{m.p.kosinski@@gmail.com}
@@ -27,9 +27,14 @@
 #'@examples
 #'
 #' library(survival)
-#' fit <- coxph(Surv(futime, fustat) ~ age + ecog.ps, data=ovarian)
+#' fit <- coxph(Surv(futime, fustat) ~ age + ecog.ps + rx, data=ovarian)
 #' cox.zph.fit <- cox.zph(fit)
+#' # plot all variables
 #' ggcoxzph(cox.zph.fit)
+#' # plot all variables in specified order
+#' ggcoxzph(cox.zph.fit, var = c("ecog.ps", "rx", "age"))
+#' # plot specified variables in specified order
+#' ggcoxzph(cox.zph.fit, var = c("ecog.ps", "rx"))
 #'
 #'@describeIn ggcoxzph Graphical Test of Proportional Hazards using ggplot2.
 #'@export
@@ -135,7 +140,7 @@ ggcoxzph <- function (fit, resid = TRUE, se = TRUE, df = 4, nsmo = 40, var,
 
 
   }) -> plots
-  names(plots) <- dimnames(yy)[[2]]
+  names(plots) <- var
   class(plots) <- c("ggcoxzph", "list")
   attr(plots, "global_pval") <- x$table["GLOBAL", 3]
   plots
